use rand::seq::SliceRandom;
use rand::rng;
use serde::Deserialize;
use serde::Serialize;

use crate::card::Card;
use crate::card::Suit;

#[derive(Debug, Serialize, Deserialize, PartialEq, Eq, Hash, Clone)]
pub struct Deck {
    pub deck: Vec<Card>,
}

impl Deck {
    pub fn new() -> Self {
        // possible values go from 0 to 8 because Durak is played
        // without cards with a value lower than 6 in real life
        // and ace is the highest card
        let values: [u8; 9] = [0, 1, 2, 3, 4, 5, 6, 7, 8];
        let suits: [Suit; 4] = [Suit::Hearts, Suit::Diamonds, Suit::Clubs, Suit::Spades];

        let mut deck_cards: Vec<Card> = Vec::new();

        for suit in suits {
            for value in values {
                let card = Card::new(suit, value);
                deck_cards.push(card);
            }
        }

        Deck {
            deck: deck_cards,
        }
    }

    pub fn get_deck(self) -> Vec<Card> {
        self.deck
    }

    pub fn shuffle(mut self) -> Self {
        let mut rng = rng();
        self.deck.shuffle(&mut rng); // shuffle modifies in place

        self
    }

    pub fn draw(mut self, n: u8) -> Vec<Card> {
        if n as usize > self.deck.len() {
            let drawn: Vec<Card> = self.deck
                .drain(0..)
                .collect();
            drawn
        } else {
            let drawn: Vec<Card> = self.deck
                .drain(0..n as usize)
                .collect();
            drawn
        }
    }
}
