use crate::{deck::Deck, player::Player};
use petname::Generator;
use serde::{Deserialize, Serialize};

#[derive(Debug, Serialize, Deserialize, Eq, PartialEq, Hash, Clone)]
pub struct Lobby {
    pub id: String,
    pub players: Vec<Player>,
    pub deck: Deck,
}

impl Lobby {
    pub fn new() -> Self {
        let players: Vec<Player> = Vec::new();
        let deck = Deck::new().shuffle();

        let petname = petname::Petnames::default().generate_one(2, "-").expect("no words provided");

        Lobby {
            id: petname,
            players,
            deck,
        }
    }

    pub fn join_lobby(&mut self, player: Player) {
        self.players.push(player);
    }

    pub fn get_players(self) -> Vec<Player> {
        self.players
    }

    pub fn get_deck(self) -> Deck {
        self.deck
    }

    pub fn get_id(&self) -> String {
        self.id.clone()
    }

    pub fn encode(&self) -> anyhow::Result<String> {
        let lobby_json = serde_json::to_string(self)?;

        Ok(lobby_json)
    }
}
