use petname::Generator;
use serde::{Deserialize, Serialize};
use sha256::digest;

use axum::Json;

use crate::card::Card;

#[derive(Default, Debug, Serialize, Deserialize, Eq, PartialEq, Hash, Clone)]
pub struct Player {
    pub id: String,
    pub name: String,
    pub hand: Vec<Card>,
}

impl Player {
    pub fn new() -> anyhow::Result<Self> {
        let hand_empty: Vec<Card> = Vec::new();


        let name = petname::Petnames::default().generate_one(2, "-").expect("no nomi");
        let to_digest = &name;
        let id = digest(to_digest);

        Ok(Player {
            id,
            name,
            hand: hand_empty,
        })
    }

    pub fn encode(&self) -> anyhow::Result<String> {
        let player_json = serde_json::to_string(self)?;

        Ok(player_json)
    }

    pub fn decode(player: String) -> anyhow::Result<Self> {
        let player: Player = serde_json::from_str(&player)?;

        Ok(player)
    }

    pub fn get_name(&self) -> String {
        self.name.clone()
    }

    pub fn get_hand(&self) -> Vec<Card> {
        self.hand.clone()
    }
}
